@@ -1,11 +1,12 @@
 /*
  * asmail is the AfterStep mailbox monitor
- * Copyright (c) 2002-2006 Albert Dorofeev <albert@tigr.net>
+ * Copyright (c) 2002-2007 Albert Dorofeev <albert@tigr.net>
  * For the updates see http://www.tigr.net/
  *
  * This software is distributed under GPL. For details see LICENSE file.
  */
 
+#include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -24,6 +25,8 @@
 #include "pop3.h"
 #include "imap.h"
 #include "gui.h"
+#include "config.h"
+
 
 int passwords_in_config = 0;
 
@@ -50,6 +53,19 @@ void usage() {
 	printf("\n");
 }
 
+static void err_printf(char const *tmpl, ...) {
+	va_list val;
+	int errno_;
+
+	errno_ = errno;
+	
+	va_start(val, tmpl);
+	vprintf(tmpl, val);
+	va_end(val);
+
+	errno = errno_;
+}
+
 /*
  * Set up the defaults that cannot be determined at compile time.
  */
@@ -223,7 +239,7 @@ int parse_cfg_check(char * line) {
 		printf("asmail: a line in configuration is too long, maximum length is %d.\n",
 				MAX_INPUT_LENGTH);
 		printf("asmail: Offending line (%d chars): %s\n", 
-				strlen(line), line);
+				(unsigned)strlen(line), line);
 		return(-1);
 	}
 	/* Leading and trailing space are removed */
@@ -238,7 +254,6 @@ int parse_cfg_check(char * line) {
 int parse_cfg_animate(FILE * f, int * line_counter, char * filename) {
 	char line[MAX_INPUT_LENGTH+1];
 	char key[MAX_INPUT_LENGTH+1];
-	char rest[MAX_INPUT_LENGTH+1];
 	struct pixfile * tmp_pixfile;
 
 	while ( fgets(line, MAX_INPUT_LENGTH, f) ) {
@@ -259,7 +274,7 @@ int parse_cfg_animate(FILE * f, int * line_counter, char * filename) {
 				tmp_pixfile = (struct pixfile *) 
 					malloc(sizeof(struct pixfile));
 				if ( ! tmp_pixfile ) {
-					printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
+					err_printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
 					perror("asmail: parse_cfg_animate: ");
 					return(-1);
 				}
@@ -272,7 +287,7 @@ int parse_cfg_animate(FILE * f, int * line_counter, char * filename) {
 				tmp_pixfile = (struct pixfile *) 
 					malloc(sizeof(struct pixfile));
 				if ( ! tmp_pixfile ) {
-					printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
+					err_printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
 					perror("asmail: parse_cfg_animate: ");
 					return(-1);
 				}
@@ -285,7 +300,7 @@ int parse_cfg_animate(FILE * f, int * line_counter, char * filename) {
 				tmp_pixfile = (struct pixfile *) 
 					malloc(sizeof(struct pixfile));
 				if ( ! tmp_pixfile ) {
-					printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
+					err_printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
 					perror("asmail: parse_cfg_animate: ");
 					return(-1);
 				}
@@ -301,7 +316,7 @@ int parse_cfg_animate(FILE * f, int * line_counter, char * filename) {
 					tmp_pixfile = (struct pixfile *) 
 						malloc(sizeof(struct pixfile));
 					if ( ! tmp_pixfile ) {
-						printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
+						err_printf("asmail: parse_cfg_animate: failed to allocate memory for the pixfile\n");
 						perror("asmail: parse_cfg_animate: ");
 						return(-1);
 					}
@@ -505,6 +520,9 @@ int init_mbox( struct mbox_struct ** mb ) {
 	new_mbox->mail = MAIL_NONE;
 	new_mbox->ctotal = 0;
 	new_mbox->cnew = 0;
+#ifdef HAVE_OPENSSL_SSL_H
+	strcpy(new_mbox->trustedCaDir, "");
+#endif
 	new_mbox->next = NULL;
 	*mb = new_mbox;
 	return(0);
@@ -613,7 +631,26 @@ int parse_cfg_mbox(FILE * f, int * line_counter, char * filename) {
 			} else {
 				new_mbox->flags &= ~FLAG_PERSISTENT_CONN;
 			}
+		} else if (!strcasecmp(key, "ssl")) {
+#ifdef HAVE_OPENSSL_SSL_H
+			sscanf(line, "%s %s", key, rest);
+			if ( ! strcasecmp(rest, "yes") ) {
+				new_mbox->flags |= FLAG_SSL;
+			} else {
+				new_mbox->flags &= ~FLAG_SSL;
+			}
+#else
+			printf("asmail: ssl support is not available, ignoring ssl statement\n");
+#endif
+		} else if (!strcasecmp(key, "trustedCaDir")) {
+#ifdef HAVE_OPENSSL_SSL_H
+			sscanf(line, "%s %s", key, new_mbox->trustedCaDir);
+#else
+			printf("asmail: ssl support is not available, ignoring trustedCaDir statement\n");
+#endif
 		} else if ( !strcmp(key, "}") ) {
+         if ((new_mbox->flags & FLAG_SSL) && (new_mbox->type != MBOX_IMAP))
+            printf("asmail: ssl is only supported with imap mailboxes, ignoring ssl statement\n");
 			/* config block terminator */
 			new_mbox->next = mbox;
 			mbox = new_mbox;
@@ -645,7 +682,7 @@ int parse_cfg(char * filename) {
 	char rest[MAX_INPUT_LENGTH+1];
 
 	if ( stat(filename, &f_stat) ) {
-		printf("asmail: Cannot stat the config file (%s).\n", filename);
+		err_printf("asmail: Cannot stat the config file (%s).\n", filename);
 		/* if the file does not exist, we still can run with
 		 * the default settings using $MAIL unless a config
 		 * file was given on command line. */
@@ -656,7 +693,7 @@ int parse_cfg(char * filename) {
 		return(-1);
 	}
 	if ( ( f = fopen(filename, "r") ) == NULL ) {
-		printf("asmail: Cannot open the config file (%s).\n", filename);
+		err_printf("asmail: Cannot open the config file (%s).\n", filename);
 		perror("asmail: parse_cfg");
 		return(-1);
 	}
@@ -696,7 +733,7 @@ int parse_cfg(char * filename) {
 	}
 
 	if ( fclose(f) ) {
-		printf("asmail: Cannot close the config file (%s).\n", filename);
+		err_printf("asmail: Cannot close the config file (%s).\n", filename);
 		perror("asmail: parse_cfg");
 		return(-1);
 	}
@@ -815,7 +852,7 @@ int main(int argc, char *argv[]) {
 		tmp_mbox = mbox;
 		while ( tmp_mbox ) {
 			if ( flag_verbose ) {
-				printf("[%d] ", tmp_mbox->thread);
+				printf("[%d] ", (unsigned)tmp_mbox->thread);
 				if ( tmp_mbox->status & STAT_RUN )
 					printf("R");
 				else
