@@ -2,15 +2,17 @@
 #define _GLOBALS_H_
 /*
  * asmail is the AfterStep mailbox monitor
- * Copyright (c) 2002-2005 Albert Dorofeev <albert@tigr.net>
+ * Copyright (c) 2002-2007 Albert Dorofeev <albert@tigr.net>
  * For the updates see http://www.tigr.net/
  *
  * This software is distributed under GPL. For details see LICENSE file.
  */
 
 #include <pthread.h>
+#include "config.h"
 
-#define VERSION "1.9"
+
+#define VERSION "2.1"
 #define RCFILE ".asmailrc"
 /* Watch out! The MAX_INPUT_LENGTH must be a multiple of 64! */
 #define MAX_INPUT_LENGTH 255
@@ -60,6 +62,7 @@ struct x11_set_struct {
 #define FLAG_ARRIVED		2
 #define FLAG_USE_MH_SEQ		4
 #define FLAG_PERSISTENT_CONN	8
+#define FLAG_SSL  16
 /* Mailbox types */
 #define MBOX_FILE	1
 #define MBOX_DIR	2
@@ -100,6 +103,9 @@ struct mbox_struct {
 	int mail;
 	int ctotal;
 	int cnew;
+#ifdef HAVE_OPENSSL_SSL_H
+	char trustedCaDir[MAX_INPUT_LENGTH+1];
+#endif
 	struct mbox_struct * next;
 };
 
