@@ -32,6 +32,7 @@
 #include <pthread.h>
 #include <errno.h>
 #include <time.h>
+#include <string.h>
 
 #include "globals.h"
 #include "gui.h"
@@ -227,10 +228,8 @@ void draw_window(Window win) {
 	int o = 0;
 	int n = 0;
 	int s = STAT_IDLE;
-	XSizeHints SizeHints;
 	int direction_return, ascent_return, descent_return;
 	XCharStruct overall_return;
-	int mail_status;
 
 #ifdef DEBUG
 	printf("asmail: draw_window: Redrawing the window...\n");
@@ -473,7 +472,6 @@ void x11_event()
 }
 
 void main_loop() {
-	XEvent Event;
 	int need_redraw;
 
 	check_mbox();
@@ -490,11 +488,12 @@ void main_loop() {
 #endif
 
 		if ( need_animation ) {
-			if ( Current )
+			if ( Current ) {
 				if ( Current->next )
 					Current = Current->next;
 				else
 					setup_pixmap();
+         }
 			need_redraw = 1;
 		}
 
@@ -512,7 +511,6 @@ void main_loop() {
 }
 
 void startx(void * ptr) {
-	int kicks;
 	int screen;
 	int color_depth;
 	XSizeHints SizeHints;
@@ -635,8 +633,8 @@ void startx(void * ptr) {
 		result = XParseGeometry(x11_set.geometry,
 				&SizeHints.x,
 				&SizeHints.y,
-				&SizeHints.width,
-				&SizeHints.height);
+				(unsigned int *)&SizeHints.width,
+				(unsigned int *)&SizeHints.height);
 		if (result & WidthValue) {
 			x_size_forced = 1;
 			winsize.x = SizeHints.width;
