@@ -20,10 +20,15 @@
 #include <sys/socket.h>
 #include <netdb.h>
 #include <netinet/in.h>
+#ifdef HAVE_OPENSSL_SSL_H 
+#include <openssl/ssl.h>
+#include <openssl/bio.h>
+#endif
 #ifdef __QNX__
 #include <sys/select.h> /* for select() et. al. */
 #endif
 
+#include "config.h"
 #include "socklib.h"
 
 #define S_LIBRARY
@@ -57,14 +62,35 @@ SOCKET *Sopen(void) {
    
    sp->sinlen = sizeof(sp->sin);
    sp->bindflag = SOCKET_RESET;
+
+#ifdef HAVE_OPENSSL_SSL_H 
+   /* ssl is initialized after the tcp connection is opened */
+   sp->ssl = NULL;
+   sp->sslBufSize = 0;
+#endif
    
    return sp;
 }
 
 int Sclose(SOCKET *sp) {
    int sd;
+
+   if (!sp)
+      return -1;
    
    sd = sp->sd;
+#ifdef HAVE_OPENSSL_SSL_H 
+   if (sp->ssl) {
+      SSL_shutdown(sp->ssl);
+      SSL_free(sp->ssl);
+   }
+
+   if (sp->sslBufSize) {
+      free(sp->sslBufMalloc);
+      sp->sslBufSize = 0;
+   }
+#endif
+
    free(sp);
    return close(sd);
 }
@@ -138,6 +164,59 @@ int Sclient(SOCKET *sp, char *name, int port) {
    return sp->sd;
 }
 
+#ifdef HAVE_OPENSSL_SSL_H 
+/* modify an open socket to use SSL */
+int Sslclient(SOCKET *sp, char *trustedCaDir)
+{
+   SSL_METHOD *meth;
+   SSL_CTX *ctx;
+   BIO *sbio;
+   X509 *peerCert;
+   
+   /* assert(sp contains an open socket) */
+
+   SSL_library_init();
+
+   meth = TLSv1_method();
+   ctx = SSL_CTX_new(meth);
+   if (!ctx)
+      return -1;
+
+   sp->ssl = SSL_new(ctx);
+   if (sp->ssl == NULL)
+      return -1;
+
+   /* don't check return value
+    * in case of error, verification will fail later on */
+   SSL_CTX_load_verify_locations(ctx, NULL, trustedCaDir);
+
+   sbio = BIO_new_socket(sp->sd, BIO_NOCLOSE);
+   if (!sbio) {
+      return -1;
+   }
+
+   SSL_set_bio(sp->ssl, sbio, sbio); /* read, write bio */
+
+   if (SSL_connect(sp->ssl) != 1) {
+      return -1;
+   }
+
+   peerCert = SSL_get_peer_certificate(sp->ssl);
+   if (!peerCert) {
+      return -1;
+   }
+
+   X509_free(peerCert);
+
+   if(SSL_get_verify_result(sp->ssl) != X509_V_OK) {
+      printf("asmail warning: could not verify server certificate\n");
+   }
+
+   return sp->sd;
+}
+#endif
+
+
 /* Read a maximum of n-1 characters from sd until a newline or
    a '\0' is received. string stores the input WITHOUT a ter-
    minating newline. Returns number of read characters, without
@@ -197,3 +276,70 @@ size_t Swrite(int sd, char *string)
 {
    return write(sd, string, strlen(string));
 }
+
+
+#ifdef HAVE_OPENSSL_SSL_H 
+/* behaves exactly like Sread() above */
+size_t Sslread(SOCKET *s, char *string, int n, int timeout)
+{
+   size_t sizeRead;
+   fd_set rfds;
+   struct timeval tv;
+   int ret;
+
+   if (s->ssl == NULL)
+      return -1;
+
+   if (!s->sslBufSize) {
+      /* empty buffer: read from SSL */
+
+      FD_ZERO(&rfds);
+      FD_SET(s->sd, &rfds);
+
+      tv.tv_sec = timeout;
+      tv.tv_usec = 0;
+
+      ret = select((s->sd)+1, &rfds, NULL, NULL, &tv);
+      if (ret > 0) {
+         s->sslBufMalloc = malloc(BUFSIZ * sizeof(char));
+         s->sslBuf = s->sslBufMalloc;
+         if (!s->sslBuf)
+            return -1;
+         s->sslBufSize = SSL_read(s->ssl, s->sslBuf, BUFSIZ);
+         if (s->sslBufSize <= 0) {
+            free(s->sslBufMalloc);
+            return -1;
+         }
+      }
+      else
+         return 0;
+   }
+
+   /* read from buffer */
+   sizeRead = 0;
+   while (s->sslBufSize-- && (sizeRead++ < n-1)) {
+      if ( ((*string++ = *(s->sslBuf)++) == 0x0d) &&
+            (*(s->sslBuf)++ == 0x0a) ) {
+         s->sslBufSize--;     /* for the 0x0a we just read */
+         string--;   /* undo 0x0d in target string */
+         sizeRead--;
+         break;
+      }
+   }
+
+   if (!s->sslBufSize)
+      free(s->sslBufMalloc);
+
+   *string = 0x0;
+   return sizeRead;
+}
+
+size_t Sslwrite(SOCKET *s, char *string)
+{
+   if (s->ssl == NULL)
+      return -1;
+
+   return SSL_write(s->ssl, string, strlen(string));
+}
+#endif
+
