@@ -13,8 +13,13 @@
 
 #define MG_SOCKLIB_H
 
+#include "config.h"
 #include <netinet/in.h>
 #include <sys/types.h>
+#ifdef HAVE_OPENSSL_SSL_H
+#include <openssl/ssl.h>
+#include <openssl/bio.h>
+#endif
 
 #ifdef __QNX__
 #define O_NDELAY O_NONBLOCK
@@ -29,9 +34,14 @@
 
 typedef struct {
    struct sockaddr_in sin;	/* sin and sinlen store address and length	*/
-   int sinlen;				/* information of the server socket			*/
+   socklen_t sinlen;				/* information of the server socket			*/
    int bindflag;			/* used to ensure bind() is only used once	*/
    int sd;					/* Socket descriptor						*/
+#ifdef HAVE_OPENSSL_SSL_H
+   SSL *ssl;            /* descriptor for ssl connection */
+   char *sslBufMalloc, *sslBuf;  /* ssl read buffer */
+   unsigned int sslBufSize;      /* its current size */
+#endif
 } SOCKET;
 
 SOCKET *Sopen(void);
@@ -40,3 +50,8 @@ int Sserver(SOCKET *, int, int);
 int Sclient(SOCKET *, char *, int);
 size_t Sread(int sd, char *string, int n, int timeout);
 size_t Swrite(int sd, char *string);
+#ifdef HAVE_OPENSSL_SSL_H
+int Sslclient(SOCKET *, char *trustedCaDir);
+size_t Sslread(SOCKET *s, char *string, int n, int timeout);
+size_t Sslwrite(SOCKET *s, char *string);
+#endif
